################################################################################
#
#    Script for use with the statistical software R
#
#    This software comes with no warranty of any kind.
#
################################################################################

# Clear workspace
rm(list=ls(all=TRUE))


# Load packages and subroutines
library(deSolve)
source("rates.R")

################################################################################
#
#    Simulate prevalence of undiagnosed and diagnosed disease
#
################################################################################



rhs <- function(a, y, parms) 
# right hand side of the differential equation
{
   with(as.list(parms), 
   {
      t  <- t0 + a
      l0 <- lambda_0(t, a) 
      l1 <- lambda_1(t, a)
      m  <- mort(t, a)
      m1 <- m * ifelse(t <= 75, 3.5, 3)
      m2 <- m * ifelse(t <= 75,   0, 2.5)

      # next line is the differential equation
      dy <- c( -((l1+l0)+(m1-m))*y[1] -     l0*y[2] + l0,
                              l1*y[1] - (m2-m)*y[2])
      list(dy)
   }   
       )
}


# simulated time period and age span
age   <- 0:105
years <- 1900 + 0:115

# prevalences (undiagnosed and diagnosed)
prev1 <- matrix(data = 0, nrow = length(years), ncol = length(age))
prev2 <- matrix(data = 0, nrow = length(years), ncol = length(age))

#this integration takes a few minutes due to the small step size
for(t0 in seq(-50, 55, by = 1)){
   parms <- c(t0 = t0)
   a_    <- seq(60, 105, by = 0.02)
   y_    <- c(0, 0)
   x     <- c(p = y_)
   out   <- as.data.frame(deSolve::rk4(x, a_, rhs, parms))
   for(aa in 60:min(105, max(115-t0, 60))){
      prev1[1+t0+aa, 1+aa] <- out$p1[1+(aa-60)*50]
      prev2[1+t0+aa, 1+aa] <- out$p2[1+(aa-60)*50]
   }
}

# Simulation is done: prev1 and prev2 contain the results      

# Figures: Prevalence of _undiagnosed_  and _diagnosed_ disease in Lexis space ###########
age   <- 50:105
years <- 55:115

# _undiagnosed_

prev  <- prev1[years+1, age+1]

layout(matrix(data=c(1,2), nrow=1, ncol=2), widths=c(4,1), heights=c(1,1))
ramp <- colorRamp(c("white", "red"))    #prev1
ColorRamp <- rgb( ramp(seq(0, 1, length = 255)), max = 255)
ColorLevels <- seq(min(prev), max(prev), length=length(ColorRamp))

# Data Map
par(mar = c(5,4,4,2))
image(x = years, y = age, z = prev, col = ColorRamp, 
        xlab = "Time (years)", ylab = "Age (years)")
contour(x = years, y = age, z = prev, add = TRUE, labcex = 0.8)

# Sidebar
par(mar = c(5,4,4,2))
image(1, ColorLevels,
     matrix(data=ColorLevels, ncol=length(ColorLevels),nrow=1),
     col=ColorRamp,
     xlab="",ylab="", xaxt="n")

# _diagnosed_

prev  <- prev2[years+1, age+1]


layout(matrix(data=c(1,2), nrow=1, ncol=2), widths=c(4,1), heights=c(1,1))
ramp <- colorRamp(c("white", "orange")) #prev2
ColorRamp <- rgb( ramp(seq(0, 1, length = 255)), max = 255)
ColorLevels <- seq(min(prev), max(prev), length=length(ColorRamp))

# Data Map
par(mar = c(5,4,4,2))
image(x = years, y = age, z = prev, col = ColorRamp, 
        xlab = "Time (years)", ylab = "Age (years)")
contour(x = years, y = age, z = prev, add = TRUE, labcex = 0.8)

# Sidebar
par(mar = c(5,4,4,2))
image(1, ColorLevels,
     matrix(data=ColorLevels, ncol=length(ColorLevels),nrow=1),
     col=ColorRamp,
     xlab="",ylab="", xaxt="n")

################################################################################
#
#    Solve the inverse problem *directly*
#
################################################################################


getInc <- function(dt, p_t1, p_t2, p_t, m_t){
   #m_t[ ,3]  is general mortality
   dp1   <- (p_t2[(71:101) + dt/2, 1] - p_t1[(71:101) - dt/2, 1])/dt
   dp2   <- (p_t2[(71:101) + dt/2, 2] - p_t1[(71:101) - dt/2, 2])/dt

   l1    <- (dp2 + (m_t[70:100, 2] - m_t[70:100, 3])*p_t[71:101, 2])/p_t[71:101, 1]
   l0    <- (dp1 + (l1 + m_t[70:100, 1] - m_t[70:100, 3])*p_t[71:101, 1])/(1-p_t[71:101,1] - p_t[71:101,2])
   return(cbind(l0, l1))
}

mm_ <- sapply(1:100, FUN = mort, t=100)
myL <- getInc(2, cbind(prev1[100, ], prev2[100, ]), cbind(prev1[102, ], prev2[102, ]),
                 cbind(prev1[101, ], prev2[101, ]), cbind(3*mm_, 2.5*mm_, mm_))

# visualise the results of the direct inversion
par(mfrow = c(1, 2))
matplot(70:100, myL[ ,1], type = "l", lty = 1, main = "Normal to Undiagnosed", xlab = "Ages (years)", ylab = "Incidence", col = "grey")
matplot(70:100, sapply(70:100, FUN = lambda_0, t = 100), type = "l", lty = 2, col = "blue", add = TRUE)
legend("topleft", legend=c("True", "Reconstructed"),
                           lty = c(2, 1), col = c("blue", "grey"))


matplot(70:100, myL[ ,2], type = "l", lty = 1, main = "Undiagnosed to Diagnosed", xlab = "Ages (years)", ylab = "Incidence", col = "grey")
matplot(70:100, sapply(70:100, FUN = lambda_1, t = 100), type = "l", lty = 2, col = "blue", add = TRUE)
legend("topleft", legend=c("True", "Reconstructed"),
                           lty = c(2, 1), col = c("blue", "grey"))

#Relative errors
abs1 <- abs(myL[ ,1] - sapply(70:100, FUN = lambda_0, t = 100))
abs2 <- abs(myL[ ,2] - sapply(70:100, FUN = lambda_1, t = 100))
summary(abs1/sapply(70:100, FUN = lambda_0, t = 100))
summary(abs2/sapply(70:100, FUN = lambda_1, t = 100))



################################################################################
#
#    Solve the inverse problem via least squares method
#
################################################################################


library(nloptr)

kq <- function(x, dt, p_t1, s_t1, p_t2, s_t2, m_t){
   #start value: prev at t* = 0.5 (t1 + t2)
   p_t      <- 0.5 * (p_t1[(71:101) - dt/2, ] + p_t2[(71:101) + dt/2, ])

   l0       <- x[ 1:31]
   l1       <- x[32:62]
   dp1      <- - (l1 + m_t[70:100, 1] - m_t[70:100, 3]) * p_t[ ,1] + l0 * (1-p_t[ ,1]-p_t[ ,2])
   dp2      <- - (m_t[70:100, 2] - m_t[70:100, 3]) * p_t[ ,2] + l1 * p_t[ ,1]
   est_p_t2 <- p_t + dt/2 * cbind(dp1, dp2)
   est_p_t1 <- p_t - dt/2 * cbind(dp1, dp2)

   return(sum(((p_t1[(71:101)-dt/2, ] - est_p_t1)/s_t1)^2) + sum(((p_t2[(71:101)+dt/2, ] - est_p_t2)/s_t2)^2))
}


#settings for lbfgs
opts <- list(xtol_rel=1.0e-8)
res  <- lbfgs(c(myL[ ,1], myL[ ,2]), fn = kq,  control = opts,
                                                    dt = 2, 
                                                  p_t1 = cbind(prev1[100, ], prev2[100, ]), 
                                                  s_t1 = rep(1, 31), 
                                                  p_t2 = cbind(prev1[102, ], prev2[102, ]), 
                                                  s_t2 = rep(1, 31), 
                                                   m_t = cbind(3*mm_, 2.5*mm_, mm_))

# visualise the results of the least squares inversion
par(mfrow = c(1, 2))
matplot(70:100, res$par[1:31], type = "l", lty = 1, main = "Normal to Undiagnosed", xlab = "Ages (years)", ylab = "Incidence", col="grey")
matplot(70:100, sapply(70:100, FUN = lambda_0, t = 100), type = "l", lty = 2, col = "blue", add = TRUE)
legend("topleft", legend=c("True", "Reconstructed"),
                           lty = c(2, 1), col = c("blue", "grey"))


matplot(70:100, res$par[32:62], type = "l", lty = 1, main = "Undiagnosed to Diagnosed", xlab = "Ages (years)", ylab = "Incidence", col="grey")
matplot(70:100, sapply(70:100, FUN = lambda_1, t = 100), type = "l", lty = 2, col = "blue", add = TRUE)
legend("topleft", legend=c("True", "Reconstructed"),
                           lty = c(2, 1), col = c("blue", "grey"))

#Relative errors
abs1 <- abs(res$par[ 1:31] - sapply(70:100, FUN = lambda_0, t = 100))
abs2 <- abs(res$par[32:62] - sapply(70:100, FUN = lambda_1, t = 100))
summary(abs1/sapply(70:100, FUN = lambda_0, t = 100))
summary(abs2/sapply(70:100, FUN = lambda_1, t = 100))

